function [M_min Error_Curve] = SF_spectral_shaping(X,W,H,M_ini,b,Error_Curve,obj_func)
% SF_spectral_shaping(X,W,H,M_ini,b,Error_Curve)
%
% Implements the spectral shaping algorithm, which gives to each pitch (phi) 
% of each instrument (d) a multiplicative factor in order to catch timbre 
% differences that depends on pitch. 
% Tensor M aims the reduction of MSE error.
%
% INPUT
% X: a matrix containing the complex spectrogram (linear or log)
% W: basis tensor resulted from the factorization
% H: weigths tensor resulted from the factorization
% M_ini: incial estimation for tensor M
% b: log-frequency resolution (12 for semitone, 24 for quartertone,...). Use b = 0 for NMF2D
%
%	OUTPUT
% M_min: Spectral shaping tensor
%
% Author: Alan F. Tygel
% Last Modified: 16/09/2009


aux = find(Error_Curve == 0);
Error_Curve = Error_Curve(1:aux(1)-1);

threshold = 10e-6; % error difference to achieve
max_adapt_it = 40; % maximum number of iterations

TAU = size(W,3);
PHI = size(H,3);
D = size(W,2);

I = size(W,1);
J = size(H,2);

if (M_ini==0)
	M = ones(I,TAU,PHI,D);
else
	M = M_ini;
end

M_min = ones(I,TAU,PHI,D);

E = 1e10;
E_min = 1e11;
it = 1;

if(b ~= 0) % LNMF2D
	X = abs(X);
	X = X(1:end/2+1,:);
	X = X./max(max(X));
	thres = 1e-5;
	V = X +thres;
	V = V./max(max(V));
else % NMF2D
	V = abs(X) + 10e-9;
	V = V./max(max(V));
end

while(1==1)

%reconstruction
Lambda = zeros(I,J);

if(b ~= 0) % LNMF2D
	for d=1:D
			for tau = 1:TAU
					for phi = 1:PHI
							Lambda = Lambda + (SF_v_linear_shift(W(:,d,tau),phi-1,b).*M(:,tau,phi,d))*SF_h_shift(H(d,:,phi),tau-1);
					end
			end
	end
else % NMF2D
	for d=1:D
			for tau = 1:TAU
					for phi = 1:PHI
							Lambda = Lambda + (SF_v_shift(W(:,d,tau),phi-1).*M(:,tau,phi,d))*SF_h_shift(H(d,:,phi),tau-1);
					end
			end
	end
end
%Lambda = Lambda/max(max(Lambda));

%  figure
%  subplot(1,2,1)
%  imagesc(Lambda)
%  subplot(1,2,2)
%  imagesc(V)

if(strcmp(obj_func,'mse'))
    E_n = sum(sum((V-Lambda).^2));
else
    div = V./Lambda;
    E_n = sum(sum(V(:,:).*log(div(:,:)) - V(:,:) + Lambda(:,:)));
end

Error_Curve = Error_Curve(:);
Error_Curve = [Error_Curve; E_n];

fprintf('Error: %d\n',E_n);

if(E_n < E_min)
	M_min = M;
	E_min = E_n;
end

if((abs(E_n-E)<threshold) || (it > max_adapt_it))
	break
end

E = E_n;

if(strcmp(obj_func,'mse'))
    for d = 1:D, for tau = 1:TAU, for phi = 1:PHI, for i = 1:I,

        if(i > (phi-1))
            j = 1:J;		
            h_aux = SF_h_shift(H(d,:,phi),tau-1);
            A = sum(V(i,j).*h_aux);
            B = sum(Lambda(i,j).*h_aux);
            M(i,tau,phi,d) = M(i,tau,phi,d)*(A/(B+eps));
        end
    end,end,end,end 
else
    for d = 1:D, for tau = 1:TAU, for phi = 1:PHI, for i = 1:I,
        if(i > (phi-1))
            j = 1:J;		
            h_aux = SF_h_shift(H(d,:,phi),tau-1);
            A = sum(V(i,j)./Lambda(i,j).*h_aux);
            B = sum(h_aux);
            M(i,tau,phi,d) = M(i,tau,phi,d)*(A/(B+eps));
            
        end    
    end,end,end,end 

end

%  multiplicacoes: D x TAU x PHI X N X (2M)
%  soma: D x TAU x PHI X N X (2M+2)

%  M = M/max(max(max(max(abs(M)))));
%  M_(:,:) = M(:,1,:);
%  imagesc(M_)
%  pause(.1)

it = it +1;

end
