function [error_c SDR PEAQ match_matrix_SDR match_matrix_PEAQ] = SoundFact_main(audio_in,varargin)

% Syntax:  SoundFact_main(audio_in,'parameter1 name',value,'parameter2 name',value, ...)
%
% SoundFact can be called with numerous arguments.
% Arguments are given in parameter pairs, so that first argument is
% the name of the parameter and the next argument is the value for
% that parameter. Parameter pairs can be given in any order.
%
% audio_in: (string)    name of the audio file. The audio file must be
% inside audio_in folder, inside a folder with the name of the audio file.
% The parameter must not contain .wav at the end.
% Example: if the audio file is called my_music.wav, it should be placed
% in SoundFact/audio_in/my_music/my_music.wav and audio_in parameter must
% be valued as 'my_music'.
%
%
% PARAMETERS:
%
% Parameter name        Values and description
%
%======================================================================
% -- Basic options: (REQUIRED)
%
% 'd'	                (int) Number of instruments in which sound will be
%			decomposed.
%
% 'tau'	                (int) Number of frames for instruments fingerprint.
%
% 'phi'	                (int) Number of possible shifts in frequency. Ex.:
%			Choose 10 if you want 5 semitones range using b = 24.
%
% 'b' (OPTIONAL)	(int) Number of bins for each octave. Ex.: 24, for 
%			quartertone resolution.
%			
%
%======================================================================
% -- Choosing the algorithms to be used: (REQUIRED)
%
% 'analysis'            (string) Chooses the time-frequency analysis
%                       to be used. 'cqt' uses logarithmic frequency, 
%												while 'fft' uses linear frequency. 'fft2log' uses
%												fft with a logarithm mapping			
%
% 'processing'           (string) Spectrogram Processing methods:
%												'ifft','mask','cross' ,'wiener','bin_mask','no_synth'
%
% 'synthesis'           (string) Phase estimation methods:
%												'ifft'(default),'GL','RTISI','RTISI-LA'
%
%
%======================================================================
% -- Spectral adaptation options: (OPTIONAL)
%
% 'spectral_adaptation' (string) Chooses wheater to use or not the 
%                       spectral enhancement. 'on' , 'off' or 'load' (Default: 'off')
%
%======================================================================
% -- Factorization algorithms options: (OPTIONAL)
%
% 'max_it'              (int) Maximum number of iterations
%                       to be used (Default: 250)
%
% 'delta'               (number) Stop criteria. (Default: 10e-3)
%
% 'obj_func'            (string) 'mse' : Mean Square Error
%                       'kld': Kullback-Leibler Divergence (Default: 'kld')
%
% 'load_fact'           (string) 'yes', 'no' to load pre-calculated data
%
%	'W_ini','H_ini','W_ini'	(matrix) Initial values for W, H, and M. Usefull for online implementation (see SF_onLine.m)
%======================================================================
% -- Analysis algorithms options: (OPTIONAL)
%
% 'load_analysis'       (string) 'yes' or 'no' (Default: 'no')
%
%
%
% 'ws' 	                (int) Window Size. It is better to choose 
%												a power of two, such as 256, 512 or 1024 (Default: 1024)
%												Usefull only for 'analysis' == 'fft' or 'fft2log'
%
% 's' 	                (int) Window Advance. Number of samples to advanced for next frame.
%												ws-s is the frame overlapping. (Default: ws/2)
%												Usefull only for 'analysis' == 'fft' or 'fft2log'
%
% 'fft_size' 	         (int) FFT Size. It is better to choose 
%												a power of two, such as 256, 512 or 1024 (Default: 1024)
%		       						  WARNING: must be greater or equal to ws.
%                       Usefull only for 'analysis' == 'fft'
%
% 'fmin'               (number) lowest frequency to be considered. (Default: 50Hz)
%												Usefull only for 'analysis' == 'cqt'
%
% 'fmax'               (number) highest frequency to be considered. (Default: Fs/2)
%												Usefull only for 'analysis' == 'cqt'
%
% 'hopsize'						 (number) hopsize to be used in cqt algorithm
%
%
%======================================================================
% -- Synthesis options: (OPTIONAL)
%
% 'out_name'           (string) Name of separated output. Will be followed by _X,
%			where X is the number of the source.(Default: Sep)
%
% 'mapping'            (string) Type of log-linear mapping. For use with CQT analysis.,
%		       Options: 'transpose' or 'psinverse' (Default: 'transpose')
%
%======================================================================
% -- Optimization Display : (OPTIONAL)
%
% 'disp_targ'		(string) Display the value of the target function for each iteration.
%                   'on' or 'off' (Default: 'on')
% 'disp_plot'		(string) In each iteration, plot the resulting decomposition. Warning: may
%                   significantly slow the algorithm (Default: 'off')
%   
% 'disp_all'		(string) Display all plots for analysis, synthesis, convergence curve, etc...
%                   Warning: may significantly slow the algorithm 'on' or 'off' (Default: 'off')
%
%======================================================================
% -- Separation Quality Evaluation Display (using BSS_EVAL): (OPTIONAL)
%
% 'bss_eval'    (string) 'yes' or 'no' for quality evaluation using bss_eval 
%                toolbox. (Default: 'no')
%
%

% Examples:
%
%
% See also: 


% Author:   Alan Freihof Tygel
% History:  05.01.2009  file created
%           18.01.2009  working: analysis (cqt,fft,fft mapped in log,);
%           factorization (nmf2d and lnmf2d using kld or mse) and synthesis
%           (ifft, mask, cross, wiener, bin_mask, cancelation)
%
%			06.02.2009  bss_eval + alignment working
%			test_all script testing every possible option
%
%           02.05.2009  Creating Online Version
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Setting Defaults

path_name = sprintf('../audio_in/%s/%s',audio_in,audio_in);
filename = sprintf('%s.wav',path_name);
[x Fs Nb] = wavread(filename);

if(size(x,2) == 2)
  x = (x(:,1) + x(:,2))/2;
  warning('Warning: converting stereo into mono');
end

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Setting Defaults

max_it = 250;
delta  = 10e-3;
WS = 1024;
S = WS/2;
FFT_size = 1024;
fMin = 50;
fMax = Fs/2;
b = 24;
out_name = 'Sep';
mapping = 'transpose';
load_analysis = 'no';
obj_func = 'kld';
load_fact = 'no';
spectral_adaptation = 'off';
disp_all = 'off';
M = 0;

global disp_targ disp_plot

disp_targ = 'on';
disp_plot = 'off';

bss_eval = 'no';
W_ini = 0;
H_ini = 0;
M_ini = 0;
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Setting required arguments to zero
analysis = 0;    
processing = 0;
synthesis = 'ifft';
D = 0;
TAU = 0;
PHI = 0;



%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Read the parameters

if (rem(length(varargin),2)==1)
  error('Optional parameters should always go by pairs');
else
  for i=1:2:(length(varargin)-1)
    if ~ischar (varargin{i}),
      error ('Unknown type of optional parameter name (parameter names must be strings).Parameter number: %d',i);
    end
    
    % change the value of parameter
    switch lower (varargin{i})
     case 'analysis'
      analysis = lower (varargin{i+1});
     case 'processing'
      processing = lower (varargin{i+1});
     case 'synthesis'
      synthesis = lower (varargin{i+1});
     case 'd'
      D = (varargin{i+1});
     case 'tau'
      TAU = varargin{i+1};
     case 'phi'
      PHI = varargin{i+1};
     case 'b'
      b = lower (varargin{i+1});
     case 'max_it'
      max_it = varargin{i+1};
     case 'delta'
      delta = varargin{i+1};
     case 'load_analysis'
      load_analysis = varargin{i+1};
     case 'ws'
      WS = (varargin{i+1}); S = WS/2;
     case 's'
      S = (varargin{i+1});
     case 'fmin'
      fMin = varargin{i+1};
     case 'fmax'
      fMax = varargin{i+1};
     case 'out_name'
      out_name = varargin{i+1};
     case 'disp_targ'
      disp_targ = varargin{i+1};
     case 'disp_plot'
      disp_plot = varargin{i+1};
     case 'mapping'
      mapping = lower (varargin{i+1} );
     case 'fft_size'
      FFT_size = varargin{i+1};
     case 'obj_func'
      obj_func = varargin{i+1};
     case 'load_fact'
      load_fact = varargin{i+1};
     case 'bss_eval'
      bss_eval = varargin{i+1};
     case 'hopsize'
      hopsize = varargin{i+1};
     case 'w_ini'
      W_ini = varargin{i+1};
     case 'h_ini'
      H_ini = varargin{i+1};
     case 'm_ini'
      M_ini = varargin{i+1};
     case 'spectral_adaptation'
      spectral_adaptation = varargin{i+1};
     case 'disp_all'
      disp_all = varargin{i+1};
    otherwise 
			varargin{i}
      error('Unknown parameter name')
    end
  end
end


%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Check mandatory parameters

if (analysis == 0)
  error('Required argument not set (analysis)')
end
if (processing == 0)
  error('Required argument not set (processing)')
end
if (D == 0)
  error('Required argument not set (D)')
end
if (TAU == 0)
  error('Required argument not set (TAU)')
end
if (PHI == 0)
  error('Required argument not set (PHI)')
end

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Check consistency

if(WS > FFT_size)
  error('FFT_size must be greater than Window Size');
end 

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%% BEGIN %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%


%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%% Analysis %%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

switch analysis
    case 'cqt'
      audio_in_cqt = sprintf('%s_cqt',path_name);
      if(strcmp(load_analysis,'no'))
        fprintf('Starting CQT analysis\n');
%        [X C hopsize PH] = SF_fast_cqt(x,Fs,fMin,fMax);     
%        C = transpose(C);

				[X  FFT_T FFT_XX C PH] = SF_fast_cqt2(x,'SR',Fs,'maxfreq',fMax,'minfreq',fMin,'windsizmax',.1*Fs,'hopsiz',hopsize);

        save(audio_in_cqt,'X','PH','Fs','Nb','C','hopsize','FFT_T')
      else
        fprintf('Loading CQT analysis\n');
    		load(audio_in_cqt)
      end

    case 'fft'
    	audio_in_fft = sprintf('%s_fft',path_name);
      if(strcmp(load_analysis,'no'))
        fprintf('Starting FFT analysis\n');
%  				S = WS/2;
%          [X PH] = SF_fft_analysis(x,WS,FFT_size,S);
        [X PH] = SF_fft_analysis2(x,WS,FFT_size,S);

%  				subplot(2,2,1)
%  				imagesc(abs(X))
%  				subplot(2,2,2)
%  				imagesc(PH)
%  				subplot(2,2,3)
%  				imagesc(X2)
%  				subplot(2,2,4)
%  				imagesc(PH2)
%  				pause;

        save(audio_in_fft,'X','PH','Fs','Nb','WS','S')
      else
        fprintf('Loading FFT analysis\n');
    		load(audio_in_fft)
      end
      X_orig = X;

    case 'fft2log'
    	audio_in_fft2log = sprintf('%s_fft2log',path_name);

      if(strcmp(load_analysis,'no'))
        fprintf('Starting FFT2LOG analysis\n');
        [X PH] = SF_fft_analysis2(x,WS,FFT_size,S);

				X_orig = X;	
				C = SF_log_frequency_mapping(fMin, fMax, Fs, WS, FFT_size ,b);
				X = C*abs(X(1:end/2,:));

        save(audio_in_fft2log,'X','PH','Fs','Nb','WS','S','X_orig','C')
      else
        fprintf('Loading FFT analysis\n');
    		load(audio_in_fft2log)
      end


    otherwise
        error('Unknown value for analysis parameter');


end

		if(strcmp(disp_all,'on'))
			figure 
			imagesc(abs(X))
		end	

fprintf('End of analysis\n');

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%% Factorization %%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%


if((strcmp(analysis,'cqt') || strcmp(analysis,'fft2log')))

    if (strcmp(obj_func,'kld'))
      audio_in_nmf2d_kld = sprintf('%s_nmf2d_kld',path_name);
      if (strcmp(load_fact,'no'))
        fprintf('Starting NMF2D - KLD factorization\n');
%        [W H Error_Curve] = SF_nmf2d(X, TAU, PHI, D, max_it, delta,W_ini,H_ini);

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
    addpath('../SNMF2D');
    opts.costfcn = 'kl';    % Set the cost function used to be the Kulback-Leibler Divergence
    opts.lambda= .01;          
    opts.Tau=0:TAU-1;           % Defines tau shifts
    opts.Phi=0:PHI-1;           % Defines phi shifts
%      opts.plotfcn = @plot_nmf2d_in_One_fig;  % Plot temporary results with specified plotfunction
    opts.maxiter = max_it;
		opts.displaylevel = 'off';
%      figure
%  		opts.plotiter
    [W H Error_Curve] = SNMF2D(abs(X), D, opts);				
%      Error_Curve = zeros(max_it,1);
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
				save(audio_in_nmf2d_kld,'W','H','Error_Curve')
      else
        fprintf('Loading NMF2D - KLD factorization\n');
        load(audio_in_nmf2d_kld);
      end

    elseif(strcmp(obj_func,'mse'))

      audio_in_nmf2d_mse = sprintf('%s_nmf2d_mse',path_name);  
      if (strcmp(load_fact,'no'))
        fprintf('Starting NMF2D - MSE factorization\n');
%          [W H Error_Curve] = SF_nmf2d_MSE2(X, TAU, PHI, D, max_it, delta,W_ini,H_ini,0.000);
        [W H Error_Curve] = SF_nmf2d_MSE(X, TAU, PHI, D, max_it, delta,W_ini,H_ini);

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%    addpath('../SNMF2D');
%    opts.costfcn = 'ed';    % Set the cost function used to be the Kulback-Leibler Divergence
%    opts.lambda= .01;          
%    opts.Tau=0:TAU-1;           % Defines tau shifts
%    opts.Phi=0:PHI-1;           % Defines phi shifts
%    opts.plotfcn = @plot_nmf2d_in_One_fig;  % Plot temporary results with specified plotfunction
%    opts.maxiter = max_it;
%    figure
%    [W H] = SNMF2D(abs(X), D, opts);				
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

      	save(audio_in_nmf2d_mse,'W','H','Error_Curve')
      else
        fprintf('Loading NMF2D - MSE factorization\n');
        load(audio_in_nmf2d_mse)
      end

    else
        
        error('Factorization: unknown option for objective function')

    end
  
elseif (strcmp(analysis,'fft'))

    if (strcmp(obj_func,'kld'))

      audio_in_lnmf2d_kld = sprintf('%s_lnmf2d_kld',path_name);
      if (strcmp(load_fact,'no'))
        fprintf('Starting LNMF2D - KLD factorization\n');
        [W H Error_Curve] = SF_lnmf2d(X, TAU, PHI, D, b, max_it, delta,W_ini,H_ini);
        save(audio_in_lnmf2d_kld,'W','H','Error_Curve')
      else
        fprintf('Loading LNMF2D - KLD factorization\n');
        load(audio_in_lnmf2d_kld)
      end

    elseif(strcmp(obj_func,'mse'))

      audio_in_lnmf2d_mse = sprintf('%s_lnmf2d_mse',path_name);        
      if (strcmp(load_fact,'no'))
        fprintf('Starting LNMF2D - MSE factorization\n');
        [W H Error_Curve] = SF_lnmf2d_MSE(X, TAU, PHI, D, b, max_it, delta,W_ini,H_ini);

        save(audio_in_lnmf2d_mse,'W','H','Error_Curve')
      else
        fprintf('Loading LNMF2D - MSE factorization\n');
        load(audio_in_lnmf2d_mse)
      end
    else

        error('Factorization: unknown option for objective function')

    end

else
      error('Factorization: Unknown value for factorization parameter');

end

if(strcmp(disp_all,'on'))
	figure 
	plot(Error_Curve(3:end))
end	

fprintf('End of factorization\n');



%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%% Spectral Adaptation %%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

if(strcmp(spectral_adaptation,'on'))
	if((strcmp(analysis,'cqt') || strcmp(analysis,'fft2log')))
		[M Error_Curve] = SF_spectral_shaping(X,W,H,M_ini,0,Error_Curve,obj_func);
	elseif (strcmp(analysis,'fft'))
		[M Error_Curve] = SF_spectral_shaping(X,W,H,M_ini,b,Error_Curve,obj_func);
	end

	if((strcmp(analysis,'cqt') || strcmp(analysis,'fft2log')))
			if (strcmp(obj_func,'kld'))
					save(audio_in_nmf2d_kld,'W','H','Error_Curve','M')
			elseif(strcmp(obj_func,'mse'))
					save(audio_in_nmf2d_mse,'W','H','Error_Curve','M')
			end
	elseif (strcmp(analysis,'fft'))
			if (strcmp(obj_func,'kld'))
				save(audio_in_lnmf2d_kld,'W','H','Error_Curve','M');
			elseif(strcmp(obj_func,'mse'))
				save(audio_in_lnmf2d_mse,'W','H','Error_Curve','M');    
			end
	end

elseif(strcmp(spectral_adaptation,'off'))
	M = ones(size(W,1),TAU,PHI,D);
end



if(strcmp(disp_all,'on'))
	figure 
	plot(Error_Curve(3:end))
end	



%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%Reconstruction and Log to linear Spectrogram mapping %%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

if(strcmp(synthesis,'no_synth'))
    return;
end    

fprintf('Starting Reconstruction\n')
%if (strcmp(analysis,'cqt') || strcmp(analysis,'fft2log'))

if(strcmp(analysis,'cqt'))
    Y_rec = SF_log_reconstruct(W,H,M);
		Y = cell(D);
    S = hopsize;  
    WS = size(Y_rec{1},1);
    
    X_orig = SF_cqttolinear(X,FFT_T);
%    imagesc(abs(X_orig))
%    pause;
    for d = 1:D
%        imagesc(abs(Y_rec{d}.*exp(j*PH)))
%        pause;
        SF_fast_cqt_synth2(X,hopsize,Fs,Nb,'xxx',FFT_T);
        Y{d} = SF_cqttolinear(Y_rec{d}.*exp(j*PH),FFT_T);                 
%         imagesc(abs(Y{d}))
%        pause
    end
end

if strcmp(analysis,'fft2log')

    Y_rec = SF_log_reconstruct(W,H,M);
    Y = cell(D);
    for d = 1:D
      if (strcmp(mapping,'transpose'))
        Y{d} = transpose(C)*Y_rec{d};         
      elseif (strcmp(mapping,'psinverse'))
        A = pinv(C.');
        Y{d} = A*Y_rec{d};
      end
      if strcmp(analysis,'fft2log')
        Y{d} = [Y{d}; flipud(Y{d})]; 
      end
    end
    
elseif strcmp(analysis,'fft')
    Y = SF_lin_reconstruct(W,H,b,M);
    for d = 1:D
        Y{d} = [Y{d}; flipud(Y{d}(2:end-1,:))]; 
    end
end
fprintf('Reconstruction Finished\n')

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%% Processing / Synthesis %%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

fprintf('Starting Processing / Synthesis %s \t %s\n',processing,synthesis);


%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%auxiliary variables

% YY_CAN = cell(D);
% for d=1:D
%     YY_CAN{d} = zeros(size(Y{d}));
%     for dd=1:D
%         if(dd ~= d)
%             YY_CAN{d} = YY_CAN{d} + Y{dd} + 10e-6;
%         end
%     end
% end

YY_CROSS = zeros(size(Y{1}));
for dd =1:D,YY_CROSS = YY_CROSS+abs(Y{dd}) + 10e-6; end

YY_WIEN = zeros(size(Y{1}));
for dd =1:D,YY_WIEN = YY_WIEN+abs(Y{dd}).^2 + 10e-6; end
%end auxiliary variables
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

misi_first_time = 1;

for d=1:D
    out_pathname = sprintf('../audio_out/%s/%s_%d.wav',audio_in,out_name,d);

    
    % processing stage, enhance of spectrgram before synthesis. Mixture's
    % phase is applied to the spectrgrams
%    if(strcmp(analysis,'cqt'))
%       X_orig = abs(X);
%    end
		Yp{d} = Y{d};
		Y{d} = abs(Y{d});
    switch processing
        case 'ifft'	
						if(~strcmp(analysis,'cqt'))
            	Y{d} = Y{d}.*exp(j*PH);
						else
							Y{d} = Yp{d};
						end
        case 'mask'
    	    Y{d} = Y{d}.*X_orig;
        case 'cross'
            Y{d} = X_orig.*Y{d}./YY_CROSS;
        case 'wiener'
            Y{d} = X_orig.*Y{d}.^2./YY_WIEN;        
        case 'bin_mask'
            M = zeros(size(X_orig));
            YY = zeros([d size(X_orig)]);
            for dd=1:D,YY(dd,:,:)=Y{dd};end
            
            for k=1:size(X_orig,1)
                for l =1:size(X_orig,2)
                    [m index] = max(abs(YY(:,k,l)));
                    if(index == d)
                        M(k,l) = 1;
                    end
                end
            end
            Y{d} = M.*X_orig;     

				otherwise 
						error('Invalid Synthesis Method');
    end
    
    addpath('synthesis');
    synth_max_it = 40;
    
    switch synthesis
        case 'ifft'
%  						imagesc(abs(Y{d}))
%  						pause;
            x = SF_fft_synth(Y{d},WS,S);
%              x = teste(Y{d},S,Fs,Nb);
        case 'gl'
            [x] = GL_synthesis(Y{d},WS,S,synth_max_it);
        case 'rtisi'
            [x] = RTISI_synth(Y{d},WS,S,synth_max_it);
        case 'rtisi-la'
            [x] = RTISI_LA_synth(Y{d},WS,S,synth_max_it);						
        case 'misi'

						if (misi_first_time == 1)
							[x_estimates] = MISI_synthesis(Y,x,WS,S,synth_max_it);
							misi_first_time = 0;
						end
						x = x_estimates(d,:);
				otherwise 
						error('Invalid Synthesis Method');
    end

    wavwrite(.9*x/max(abs(x)),Fs,Nb,out_pathname);
    

end %for
fprintf('End of Synthesis\n');


%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%% Quality Evaluation%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

sources 	= cell(D);
separated = cell(D);

if(strcmp(bss_eval, 'yes'))
    for d = 1:D
        source_name = sprintf('%s_s%d',path_name,d);
        sources{d}.signal = wavread(source_name);  
        sources{d}.name = source_name;  
		
        out_pathname = sprintf('../audio_out/%s/%s_%d.wav',audio_in,out_name,d);
        separated{d}.signal = wavread(out_pathname);
        separated{d}.name = out_pathname;

    end

%   		subplot(2,2,1)
%   		plot(sources{1}.signal);		
%   		subplot(2,2,2)
%   		plot(separated{1}.signal);		
%   		subplot(2,2,3)
%   		plot(sources{2}.signal);		
%   		subplot(2,2,4)
%   		plot(separated{2}.signal);		

%  		eval_name = sprintf('../audio_out/%s/BSS_eval_results.csv',audio_in);
		eval_name = 0;
		[SDR match_matrix_SDR] = SF_BSS_eval(sources, separated,Fs,Nb,out_name,eval_name);

		warning('off');
		[PEAQ match_matrix_PEAQ] = SF_PEAQ_eval(sources, separated,Fs,Nb,out_name);
		warning('on');
		
		error_c = Error_Curve(max(find(Error_Curve~=0)));
		
end

end